@@ -36,10 +36,11 @@ class mTodos(object):
 
   __pretty_app_name__ = "mTodos"
   __app_name__ = "mTodos"
-  __version__ = "0.0.2"
+  __version__ = "0.2.0"
   __context__ = "net.khertan.mtodos"
 
   _glade_files = [
+#    'ui/mtodos.glade',
     '/usr/lib/mtodos/ui/mtodos.glade',
   ]
 
@@ -95,12 +96,14 @@ class mTodos(object):
     # 2-> Edit
     # 3-> DatePicker
 
+    self._current_category = 'All'
+
     callbackMapping = {
       # View Signal
-      "on_bNow_clicked": self._show_now,
-      "on_bNext_clicked": self._show_next,
-      "on_bFuture_clicked": self._show_future,
-      "on_bArchive_clicked": self._show_archive,
+      "on_bNow_toggled": self._show_now,
+      "on_bNext_toggled": self._show_next,
+      "on_bFuture_toggled": self._show_future,
+#      "on_bArchive_clicked": self._show_archive,
       "on_bAdd_clicked": self._add_todo,
       "on_bEdit_clicked": self._edit_todo,
       "on_bCancel_clicked": self._cancel,
@@ -112,8 +115,9 @@ class mTodos(object):
       "on_bSearch_clicked": self._open_search_diag,
       "on_bSearchCancel_clicked": self._cancel_search_diag,
       "on_bSearchValidate_clicked": self._validate_search_diag,
+      "on_bCat_changed":self._bCat_changed,
+      "on_tDisplay_row_activated":self._edit_todo
     }
-
     self._widgetTree.signal_autoconnect(callbackMapping)
 
     if self._window:
@@ -138,7 +142,7 @@ class mTodos(object):
       self.refresh()  
       self._update_virtual(1)
 
-  def _edit_todo(self,widget):
+  def _edit_todo(self,widget,*args):
     selection = self.todosListTree.get_selection()
     model, selection_iter = selection.get_selected()
     if (selection_iter):
@@ -227,21 +231,37 @@ class mTodos(object):
 #      self._tasksList.set_value(selection_iter, 5,task.get_fg_color())
 #      task.save_to_db()
 #      self.refresh()
+  def _if_all_false(self,widget):
+    if (self._widgetTree.get_widget("bNow").get_active()==False) and (self._widgetTree.get_widget("bNext").get_active()==False) and (self._widgetTree.get_widget("bFuture").get_active()==False):
+      widget.set_active(True)
+
   def _show_now(self,widget):
-    if self._widgetTree.get_widget("bNow").get_active()==True:
+    print 'now'
+    if (self._widgetTree.get_widget("bNow").get_active()==True) :
+#    if self._current_view!=1:
       self._update_view(1)
+    else:
+      self._if_all_false(self._widgetTree.get_widget("bNow"))
 
   def _show_next(self,widget):
-    if self._widgetTree.get_widget("bNext").get_active()==True:
+    print 'next'
+    if (self._widgetTree.get_widget("bNext").get_active()==True):
       self._update_view(2)
+    else:
+       self._if_all_false(self._widgetTree.get_widget("bNext"))
 
   def _show_future(self,widget):
-    if self._widgetTree.get_widget("bFuture").get_active()==True:
+    print 'future'
+    if (self._widgetTree.get_widget("bFuture").get_active()==True):
       self._update_view(3)
+    else:
+      self._if_all_false(self._widgetTree.get_widget("bFuture"))
 
   def _show_archive(self,widget):
-    if self._widgetTree.get_widget("bArchive").get_active()==True:
-      self._update_view(4)
+    #if self._widgetTree.get_widget("bArchive").get_active()==True:
+    self._current_category='All'
+    self._widgetTree.get_widget("bCat").set_active(0)
+    self._update_view(4)
 
   def _cancel(self,widget):
     self._update_virtual(1)
@@ -269,10 +289,26 @@ class mTodos(object):
     self.refresh()
 #    self._load_todoslist_bg()
 
+  def _set_category_available(self,widget,isEntry=False):
+    gtk.gdk.threads_enter()
+    model = gtk.ListStore(str)
+    widget.set_model(model)
+    if isEntry==True:
+      widget.set_text_column(0)
+    for cat in mtodos_framework.TodosList().get_categories():
+      model.append((str(cat),))
+    if isEntry==False:
+      widget.set_active(0)
+    gtk.gdk.threads_leave()
+
   def _open_todo(self,uid):
-    self._current_todo = mtodos_framework.Todo(uid)
+    threading.Thread(target=self._set_category_available,args=[self._widgetTree.get_widget("eCat"),True]).start()
+    self._current_todo = mtodos_framework.Todo(uid,default_category=self._current_category)
     self._widgetTree.get_widget("todo_status").set_active(self._current_todo.status)
     self._widgetTree.get_widget("todo_duedate").set_text(self._current_todo.due_datetime.strftime('%x'))
+    if self._current_todo.category==None:
+      self._current_todo.category='None'
+    self._widgetTree.get_widget("eCat").get_child().set_text(self._current_todo.category)
     self._widgetTree.get_widget("todo_description").get_buffer().set_text(self._current_todo.description)
     self._update_virtual(2)
 
@@ -281,6 +317,7 @@ class mTodos(object):
     self._current_todo.due_datetime = datetime.datetime.strptime(self._widgetTree.get_widget("todo_duedate").get_text(),'%x')
     start,end = self._widgetTree.get_widget("todo_description").get_buffer().get_bounds()
     self._current_todo.description = self._widgetTree.get_widget("todo_description").get_buffer().get_text(start,end)
+    self._current_todo.category = self._widgetTree.get_widget("eCat").get_child().get_text()
     self._current_todo.save_to_db()
 
   def _add_todo(self,widget):
@@ -317,26 +354,27 @@ class mTodos(object):
     self._previous_view = self._current_view
     self._current_view = view
 
+    threading.Thread(target=self._set_category_available,args=[self._widgetTree.get_widget("bCat")]).start()
     if view == 1:
       self._widgetTree.get_widget("bNow").set_active(True)
       self._widgetTree.get_widget("bNext").set_active(False)
       self._widgetTree.get_widget("bFuture").set_active(False)
-      self._widgetTree.get_widget("bArchive").set_active(False)
+#      self._widgetTree.get_widget("bArchive").set_active(False)
     elif view == 2:
       self._widgetTree.get_widget("bNow").set_active(False)
       self._widgetTree.get_widget("bNext").set_active(True)
       self._widgetTree.get_widget("bFuture").set_active(False)
-      self._widgetTree.get_widget("bArchive").set_active(False)
+#      self._widgetTree.get_widget("bArchive").set_active(False)
     elif view == 3:
       self._widgetTree.get_widget("bNow").set_active(False)
       self._widgetTree.get_widget("bNext").set_active(False)
       self._widgetTree.get_widget("bFuture").set_active(True)
-      self._widgetTree.get_widget("bArchive").set_active(False)
+#      self._widgetTree.get_widget("bArchive").set_active(False)
     elif view == 4:
       self._widgetTree.get_widget("bNow").set_active(False)
       self._widgetTree.get_widget("bNext").set_active(False)
       self._widgetTree.get_widget("bFuture").set_active(False)
-      self._widgetTree.get_widget("bArchive").set_active(True)
+#      self._widgetTree.get_widget("bArchive").set_active(True)
 
     self._load_todoslist_bg()
 
@@ -386,8 +424,8 @@ class mTodos(object):
 #
     checkboxrenderer = gtk.CellRendererToggle()
     checkboxrenderer.set_property('activatable' , True)
-    checkboxrenderer.set_property('height' , 60)
-    checkboxrenderer.set_property('width' , 60)
+    checkboxrenderer.set_property('height' , 50)
+    checkboxrenderer.set_property('width' , 50)
     checkboxrenderer.set_property('cell-background-set' , True)
 #
     checkboxrenderer.connect("toggled", self.edited_checkbox_cb, (self._todosList, 1))    
@@ -451,7 +489,7 @@ class mTodos(object):
     self._window.show()
 
     self._widgetTree.get_widget("bNow").set_active(True)
-
+#    self._widgetTree.get_widget("bCat").set_active(0)
     self._todos_loader_process = threading.Thread(target=self._set_label)
     self._todos_loader_process.start()
 
@@ -497,31 +535,41 @@ class mTodos(object):
 
   def _set_label(self):
     gtk.gdk.threads_enter()
-    self._now_label.set_markup('<b>Now</b>\n'+str(len(mtodos_framework.TodosList().get_now_todos()))+' tasks')
-    self._next_label.set_markup('<b>Next</b>\n'+str(len(mtodos_framework.TodosList().get_next_todos()))+' tasks')
-    self._future_label.set_markup('<b>Future</b>\n'+str(len(mtodos_framework.TodosList().get_future_todos()))+' tasks')
-    self._archive_label.set_markup('<b>Archive</b>\n'+str(len(mtodos_framework.TodosList().get_all_todos()))+' tasks')
+    todos_list = mtodos_framework.TodosList()
+    self._now_label.set_markup('<b>Now</b>\n'+str(len(todos_list.get_now_todos(self._current_category)))+' tasks')
+    self._next_label.set_markup('<b>Next</b>\n'+str(len(todos_list.get_next_todos(self._current_category)))+' tasks')
+    self._future_label.set_markup('<b>Future</b>\n'+str(len(todos_list.get_future_todos(self._current_category)))+' tasks')
+    self._archive_label.set_markup('<b>Archive</b>\n'+str(len(todos_list.get_all_todos(self._current_category)))+' tasks')
+    todos_list.close()
     gtk.gdk.threads_leave()
 
+  def _bCat_changed(self,widget):
+    model = widget.get_model()
+    active = widget.get_active()
+    if active < 0:
+        return None
+    self._current_category = model[active][0]
+    self._load_todoslist_bg()
+
   def _load_todoslist(self,todosList):
     gtk.gdk.threads_enter()
     todosList.clear()
     gtk.gdk.threads_leave()
     if self._current_view == 1:
-      todos_uuid_list =  mtodos_framework.TodosList().get_now_todos()
+      todos_uuid_list =  mtodos_framework.TodosList().get_now_todos(self._current_category)
     elif self._current_view == 2:
-      todos_uuid_list =  mtodos_framework.TodosList().get_next_todos()
+      todos_uuid_list =  mtodos_framework.TodosList().get_next_todos(self._current_category)
     elif self._current_view == 3:
-      todos_uuid_list =  mtodos_framework.TodosList().get_future_todos()
+      todos_uuid_list =  mtodos_framework.TodosList().get_future_todos(self._current_category)
     elif self._current_view == 4:
-      todos_uuid_list =  mtodos_framework.TodosList().get_all_todos()
+      todos_uuid_list =  mtodos_framework.TodosList().get_all_todos(self._current_category)
     todos_uuid_list.sort()
     todo = None
     for (todo_status,todo_due_datetime,todo_uuid) in todos_uuid_list:
       if self._todosList != todosList:
         return False
       todo = mtodos_framework.Todo(todo_uuid)
-      if todo.due_datetime != None:
+      if todo_due_datetime != None:
         due_datetime = datetime.datetime.strftime(todo_due_datetime,'%x')
       else:
         due_datetime = None
@@ -603,6 +651,25 @@ class mTodos(object):
     accel_group = gtk.AccelGroup ()
     self._window.add_accel_group (accel_group)
 
+    #Search
+    item = gtk.ImageMenuItem (gtk.STOCK_SELECT_ALL, accel_group)
+    item.get_child().set_text('Show all')
+    item.connect ("activate", self._show_archive)
+    item.show ()
+    self.menubar.append (item)
+
+    #Search
+    item = gtk.ImageMenuItem (gtk.STOCK_FIND, accel_group)
+    item.get_child().set_text('Search')
+    item.connect ("activate", self._open_search_diag)
+    item.show ()
+    self.menubar.append (item)
+
+    #Separator
+    item = gtk.SeparatorMenuItem ()
+    item.show ()
+    self.menubar.append (item)
+
     #Purge
     item = gtk.ImageMenuItem (gtk.STOCK_DELETE, accel_group)
     item.get_child().set_text('Purge Todos done')
@@ -652,4 +719,4 @@ def run_mtodos(app,context):
   gtk.main()
 
 if __name__ == "__main__":
-  run_mtodos(hildon.Program(),None)
\ No newline at end of file
+  run_mtodos(hildon.Program(),None)
